import os

def _get_color(color=None):
  """Get color name from pre-defined color list"""
  from .rainbow import _is_no_color
  if 'RAINBOW_LOGGER_NO_COLOR' in os.environ and os.environ['RAINBOW_LOGGER_NO_COLOR'] == 'false':
    return ''

  if _is_no_color:
    return ''

  if color is None and not color:
    raise ValueError('Must have color code')

  colors = {
    'PURPLE':'\033[96m',
    'MAGENTA':'\033[95m',
    'BLUE':'\033[94m',
    'GREEN':'\033[92m',
    'YELLOW':'\033[93m',
    'RED':'\033[91m',
    'DARKGRAY':'\033[90m',
    'GREY':'\033[0m',
    'WHITE':'\033[1m'
  }

  if color not in colors:
    raise KeyError('Use a proper color name: {}'.format(list(colors.keys())))

  return colors[color]

def _get_message(no_time=False):
  """Get message parts"""
  time = '{}%(asctime)s{}'.format(_get_color('DARKGRAY'), _get_color('GREY'))
  name = '{}%(name)-12s{}'.format(_get_color('PURPLE'), _get_color('GREY'))
  level = '%(levelname)-8s'
  message = '%(message)s'

  if 'RAINBOW_LOGGER_NO_TIME' in os.environ and os.environ['RAINBOW_LOGGER_NO_TIME'] == 'false':
    return ''

  if no_time:
    return '{} {}\t{}'.format(name, level, message)

  return '{} {} {}\t{}'.format(time, name, level, message)

def _set_level_format(level=None, color='WHITE', logging_module=None):
  """Set logging format based on level and color"""
  FORMAT = '{}{}{}'
  _logging_module = logging_module
  parsed_format = FORMAT.format(
    _get_color(color),
    _logging_module.getLevelName(level),
    _get_color('GREY')
  )

  if level is not None:
    _logging_module.addLevelName(level, parsed_format)

  return True



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['rainbow']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'rainbow-bridge-logger',
    'version': '0.5.0',
    'description': 'Improved wrapper for native logger of Python 3',
    'long_description': "\n\nRainbow Bridge Logger\n=================================\nA wrapper for the native logging module of Python.\n\n.. code-block:: python\n\n    from rainbow import RainbowLogger\n\n    # __name__ will get the current context\n    # but you can pass any text you want, for identification\n    logger = RainbowLogger(__name__)\n\n    logger.info('my info')\n    logger.warning('my warn')\n    logger.error('my error')\n    logger.debug('my debug')\n",
    'author': 'Almer Mendoza',
    'author_email': 'atmalmer23@gmail.com',
    'url': 'https://github.com/mamerisawesome/rainbow-bridge-logger',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*',
}


setup(**setup_kwargs)



"""
:Author: Almer Mendoza
:Since: 10/10/2018
:Updated:
  - 08/26/2019: Environment level configurations
"""
import os
from .helpers import _set_level_format, _get_message

_logging_module = None
_is_no_color = False

class RainbowLogger:
  """A customized logger built on top of Python's logging"""
  def __new__(self,
              name=None,
              no_time=False,
              no_color=False,
              new_logging=None,
              filepath=None,
              log_level=None,
              get_logging=False):
    import logging
    global _logging_module
    global _is_no_color

    _is_no_color = no_color
    _logging_module = logging
    if new_logging is not None:
      _logging_module = new_logging

    level_color_mapping = [
      {"level": _logging_module.DEBUG, "color": "BLUE"},
      {"level": _logging_module.INFO, "color": "GREEN"},
      {"level": _logging_module.WARN, "color": "YELLOW"},
      {"level": _logging_module.ERROR, "color": "RED"},
      {"level": _logging_module.CRITICAL, "color": "MAGENTA"}
    ]

    for l in level_color_mapping:
      _set_level_format(level=l['level'],
                        color=l['color'],
                        logging_module=_logging_module)

    if name is None:
      logger = _logging_module
    else:
      logger = _logging_module.getLogger(name)

    handler = _logging_module.StreamHandler()
    final_message = _get_message(no_time)
    formatter = _logging_module.Formatter(final_message)

    handler.setFormatter(formatter)
    log_level = log_level if log_level else _logging_module.DEBUG
    if 'RAINBOW_LOGGER_ON_WARN' in os.environ and os.environ['RAINBOW_LOGGER_ON_WARN'] == 'true':
      log_level = _logging_module.WARN

    if 'RAINBOW_LOGGER_ON_ERROR' in os.environ and os.environ['RAINBOW_LOGGER_ON_ERROR'] == 'true':
      log_level = _logging_module.ERROR

    if name is None:
      _logging_module.basicConfig(format=final_message,
                                  level=log_level)
    else:
      logger.addHandler(handler)
      logger.setLevel(log_level)

    if filepath and name:
      handler = _logging_module.FileHandler(filepath)
      _is_no_color = True
      final_message = _get_message(no_time)
      formatter = _logging_module.Formatter(final_message)
      handler.setFormatter(formatter)
      logger.addHandler(handler)

    if get_logging:
      return _logging_module

    return logger

def __init__():
  return RainbowLogger
